#!/usr/bin/perl -w

## Header:

# Sky-Jukebox for irssi -- this is a port of Sky-Script for xchat to
# the irssi client.

# Porter: Sascha Ldecke AKA Moxon <moxon@meta-x.de>


# TODO

# - change of timer values should have effect immediately, not on restart/reset
# - currently only servers[0] supported, i.e. only first server. add
#   server configurable tag
# - add removal from queue after part immediately (tell users in queue by message)
# - add timed removal from queue after quit (server hop, tell users)
# - add dcc cps detection
# - publish to script repository

## Original Header:

# Sky-Script for X-chat irc client.
# Copyright (C) 2000 by Ulrich Wiederhold

# Thanks to Joseph Elwell, Leo Cacciari and the xchat-discuss mailinglist,
# who helped me to learn perl.

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# $Id: sky-jukebox.pl,v 1.6 2002/05/04 12:42:00 chat Exp $


##### use declarations #################################################

use strict;
no strict 'refs';
use vars qw($1 $2 $3 $4 @arrays %files_with_path %config %locator_hash @allkeys @timers);

use Irssi;
use Irssi::Irc;
use MP3::Info;

my $sky_version = "0.9.12";

#######################################################################
# This must be set or the script wont compile
%config = ();
$config{'confpath'} = $ENV{'HOME'} . "/.sky-jukebox/sky-jukebox.conf";
$config{'confdir'} = $ENV{'HOME'} . "/.sky-jukebox/"; #gets the config path
$config{'scriptpath'} = $ENV{'HOME'} . "/software/sky-jukebox/sky-jukebox-$sky_version.tar.gz";
@arrays = ('offerdirs','noffersubdirs','channels','onjoin_channels','autovoice_channels','locator_channels');
$config{'nolist'} = 1;
$config{'timer'} = 300000;
$config{'slotstimer'} = 180000;
$config{'server_status'} = "off";
$config{debug} = 0;
%files_with_path = ();

@timers = ();

my $prefix = "Sky-Jukebox";
#################################################################

sky_print("\00314,8\0032Loading \0034Sky-Jukebox $sky_version \0032by Moxon AKA Sascha Ldecke\003");
sky_print("\00314,8\0032Original XChat version by Ulrich Wiederhold\003");
sky_print("\00314,8\0032To configure type \0034/settings \003");
sky_print("\00314,8\0032If you need help type \0034/skyhelp \003");

##############Serving-Part######################

Irssi::command_bind('settings', 'sky_settings_handler');
Irssi::command_bind('createlist', 'sky_create_list');
Irssi::command_bind('loadconf', 'sky_load_conf');
Irssi::command_bind('saveconf', 'sky_save_conf');
Irssi::command_bind('skyhelp', 'sky_help');
Irssi::command_bind('play', 'sky_play');
Irssi::command_bind('sendnext', 'sky_sendNext');

Irssi::command_bind('skyon', 'sky_turn_server_on');
Irssi::command_bind('skyoff', 'sky_turn_server_off');
Irssi::command_bind('skyjoin', 'sky_join_channels');

Irssi::signal_add('message public', 'sky_publicmsg_handler');
Irssi::signal_add('message private', 'sky_privmsg_to_mynick_handler');
Irssi::signal_add('ctcp msg', 'sky_notice_handler');

#IRC::add_message_handler('PRIVMSG', 'sky_autovoice');
#IRC::add_message_handler('JOIN', 'sky_join_handler');
#IRC::add_message_handler('PART', 'sky_part_handler');
#IRC::add_message_handler('QUIT', 'sky_quit_handler');

Irssi::signal_add_last('dcc error connect','sky_sendNext');
Irssi::signal_add_last('dcc error file open','sky_sendNext');
Irssi::signal_add_last('dcc error get not found','sky_sendNext');
Irssi::signal_add_last('dcc error send exists','sky_sendNext');
Irssi::signal_add_last('dcc error unknown type','sky_sendNext');
Irssi::signal_add_last('dcc error close not found','sky_sendNext');
Irssi::signal_add_last('dcc rejecting', 'sky_sendNext');
Irssi::signal_add('dcc closed', 'sky_filecounter');

&sky_turnon_timer;

#################@locator-part####################

Irssi::signal_add("message public", "sky_locator_handler");

##################################################

###########################################################################
#
# Some internal functions
#
###########################################################################

sub sky_error
{
    Irssi::print("$prefix ERROR: @_", MSGLEVEL_CLIENTCRAP);
}

sub sky_print
{
    Irssi::print("$prefix: @_", MSGLEVEL_CLIENTCRAP);
}

sub sky_debug
{
    if ($config{debug}) {
        Irssi::print("$prefix DEBUG: @_", MSGLEVEL_CLIENTCRAP);
      }
}

sub sky_notice_nick
{
    my ($nick, $msg) = @_;
    if ($config{'send_raw_status'} eq 'on') {
        srv_send_raw("NOTICE $nick :\003$config{'background_color'},2\003$config{'script_color'}$msg");
    }
    elsif ($config{'send_raw_status'} eq 'off') {                                                 
        srv_command("/msg $nick \003$config{'background_color'},2\003$config{'script_color'}$msg");
    } else {
        sky_error("Unknown state for send_raw_status: $config{'send_raw_status'}!");
    }
}

sub srv_send_raw ($)
{
    my $param = shift;
    my $server = (Irssi::servers())[0];
    if ($server) {
        $server->send_raw($param);
    } else {
        sky_error("No server found, cannot send raw: $param");
    }
}
sub srv_command ($)
{
    my $param = shift;
    my $server = (Irssi::servers())[0];
    if ($server) {
        $server->command($param);
    } else {
        sky_error("No server found, cannot send raw: $param");
    }
}
sub srv_channel_find ($)
{
    my $param = shift;
    my $server = (Irssi::servers())[0];
    if ($server) {
        return $server->channel_find($param);
    } else {
        sky_error("No server found, cannot send raw: $param");
        return ();
    }
}

###########################################################################

sub sky_turnon_timer
{
    &sky_load_conf;
    return 0;
}

sub sky_turn_server_on
{

    if ($config{'server_status'} eq 'on') {
        sky_print("Server already on.");
        return;
    }

    $config{'server_status'} = 'on';

    if ($config{'nolist'} != 1) {
        &sky_timer;
        &sky_slots_timer;
        ## [FIX] push @timers, Irssi::timeout_add($config{'timer'},"sky_offerscript", '');
        push @timers, Irssi::timeout_add($config{'timer'}, "sky_timer", '');
        push @timers, Irssi::timeout_add($config{'slotstimer'}, "sky_slots_timer", '');
        sky_print("Sky-Jukebox turned on.");
    }
    elsif ($config{'nolist'} == 1) {
        sky_print("No list available. Please check your settings and run createlist!");
    }
    return 0;
}


sub sky_turn_server_off
{
    if ($config{'server_status'} eq 'off') {
        sky_print("Server already off.");
        return;
    }

    $config{'server_status'} = 'off';
    my $tag;
    while ($tag = pop @timers) {
        sky_print("Removing timer $tag");
        Irssi::timeout_remove($tag);
    }
    sky_print("Sky-Jukebox turned off.");
    return 0;
}

sub sky_join_channels
{
    my $server = (Irssi::servers())[0];
    if ($server) {
        foreach (@{ $config{channels} }) {
            my $channel = $server->channel_find($_);
            unless ($channel && $channel->{joined}) {
                Irssi::Server::channels_join($server, $_, 'automatic');
            }
        }
    } else {
        sky_error("Cannot join channels, not connected to a server.");
    }
}


sub sky_load_conf
{
    sky_print("Loading config $config{'confpath'}");
    @allkeys = "";
    open(LOADCONF, "<$config{'confpath'}") or sky_error("Cant open confpath in load_conf. fix and restart"); # die;
    while(<LOADCONF>)
    {
        if ($_ =~ /^\s.*$|^\#.*$/)
        {
            next;
        }
        elsif (!($_ =~ /^\s.*$|^\#.*$/))
        {
            my ($key, $reststring) = split(/:/);
            push(@allkeys,$key);
            chop $reststring;
            my @values = split(" ",$reststring);
            if ("@arrays" =~ /\Q$key\E/i)
            {
                $config{$key} = \@values;
            }
            elsif ($#values < 1)
            {
                $config{$key} = $values[0];
            }
        }
    }
    close(LOADCONF);
#  foreach (keys %config)
#    {
#      no strict "refs";
#      ref($config{$_}) ? @$_ = @{$config{$_}} : $$_ = $config{$_};
#    }
    $config{'trigsong'} = "!" . "$config{'mynickname'}"; #'!FzzGrr'; #trigger to request a song
    $config{'triglist'} = "@" . "$config{'mynickname'}"; #'@FzzGrr'; #trigger to request the list
    $config{'queuelocation'} = $config{'confdir'} . "queues.log"; #gets the queuefile-path and selects the filename
    $config{'listpath'} = $config{'confdir'} . $config{'mynickname'} . "-" . $config{'listdate'} . ".txt"; #gets the offerlist-path and selects the filename
    $config{'listofpath'} = $config{'confdir'} . $config{'mynickname'} . "Path-" . $config{'listdate'} . ".txt"; #gets the pathlist-path and selects the filename
    $config{'ziplistpath'} = $config{'confdir'} . $config{'mynickname'} . "-" . $config{'listdate'} . ".zip";
    unless ($config{'startdate'} =~ m~^\d{2}-\d{2}-\d{4}$~) {
        $config{'startdate'} = `date +%d-%m-%Y`;      #gets the date
        chomp $config{'startdate'};                    #cuts \n off the end
        &sky_create_list;
    }
    if (open(CONTENT, "<$config{'listofpath'}")) {
        while (<CONTENT>){
            # [TR FIX] (my $underline_filename = $_) =~ tr/ /_/;
            my $underline_filename = $_;
            $underline_filename =~ /^.+\/(.+)/;
            my $file = $1;
            chomp $file;
            $files_with_path{$file} = $underline_filename;
        }
        open(LISTE, "<$config{'listpath'}");
        while (<LISTE>){
            if ($_ =~ /^\s.*$|^\#.*$/) {
                next;
            }
            elsif (!($_ =~ /^\s.*$|^\#.*$/)) {
                # [TR FIX] (my $filename = $_) =~ tr/ /_/;
                my $filename = $_;
                $filename = substr($filename, length($config{'trigsong'})+1);
                $locator_hash{lc $filename} = $_;
            }
            next;
            close(LISTE);
        }
        $config{'nolist'} = 0;
        close(CONTENT);
    }
    else {
        $config{'nolist'} = 1;
    }

    return 1;
}


sub sky_save_conf()
{
    open(SAVECONF, ">$config{'confpath'}.new") or sky_error("Cant open confpath.new in save_conf, fix and restart") and return;# die;
    print(SAVECONF "# Configure these values\n");
    print(SAVECONF "mynickname:$config{'mynickname'}\n");
    print(SAVECONF "offerdirs:@{$config{'offerdirs'}}\n");
    print(SAVECONF "noffersubdirs:@{$config{'noffersubdirs'}}\n");
    print(SAVECONF "channels:@{$config{'channels'}}\n");
    print(SAVECONF "maxSends:$config{'maxSends'}\n");
    print(SAVECONF "maxQueues:$config{'maxQueues'}\n");
    print(SAVECONF "timer:$config{'timer'}\n");
    print(SAVECONF "slotstimer:$config{'slotstimer'}\n");
    print(SAVECONF "maxQPN:$config{'maxQPN'}\n");
    print(SAVECONF "maxSPN:$config{'maxSPN'}\n");
    print(SAVECONF "onjoin_status:$config{'onjoin_status'}\n");
    print(SAVECONF "onjoin_channels:@{$config{'onjoin_channels'}}\n");
    print(SAVECONF "autovoice_status:$config{'autovoice_status'}\n");
    print(SAVECONF "autovoice_channels:@{$config{'autovoice_channels'}}\n");
    print(SAVECONF "locator_status:$config{'locator_status'}\n");
    print(SAVECONF "locator_channels:@{$config{'locator_channels'}}\n");
    #  print(SAVECONF "server_status:$server_status\n");
    print(SAVECONF "ziplist:$config{'ziplist'}\n");
    print(SAVECONF "debug:$config{'debug'}\n");
    print(SAVECONF "show_filesize_in_list:$config{'show_filesize_in_list'}\n");
    print(SAVECONF "show_details_in_list:$config{'show_details_in_list'}\n");
    print(SAVECONF "send_raw_status:$config{'send_raw_status'}\n");
    print(SAVECONF "netsplitlimit:$config{'netsplitlimit'}\n");
    print(SAVECONF "connection_type:$config{'connection_type'}\n");
    print(SAVECONF " \n");
    print(SAVECONF "# Color Settings:\n");
    print(SAVECONF "background_color:$config{'background_color'}\n");
    print(SAVECONF "script_color:$config{'script_color'}\n");
    print(SAVECONF "#  0 = white\n");
    print(SAVECONF "#  1 = black\n");
    print(SAVECONF "#  2 = royal-blue\n");
    print(SAVECONF "#  3 = green-light\n");
    print(SAVECONF "#  4 = red\n");
    print(SAVECONF "#  5 = dark-red\n");
    print(SAVECONF "#  6 = violett\n");
    print(SAVECONF "#  7 = orange\n");
    print(SAVECONF "#  8 = yellow\n");
    print(SAVECONF "#  9 = green\n");
    print(SAVECONF "# 10 = dark-turquoise\n");
    print(SAVECONF "# 11 = turquoise\n");
    print(SAVECONF "# 12 = blue\n");
    print(SAVECONF "# 13 = pink\n");
    print(SAVECONF "# 14 = grey-light\n");
    print(SAVECONF "# 15 = dark-grey\n");
    print(SAVECONF "# To see the colors: \"Settings -> Palette\"\n");
    print(SAVECONF " \n");
    print(SAVECONF "# Sky-Jukebox Internal Settings\n");
    print(SAVECONF "filecounter:$config{'filecounter'}\n");
    print(SAVECONF "mbsent:$config{'mbsent'}\n");
    print(SAVECONF "cpsrecord:$config{'cpsrecord'}\n");
    print(SAVECONF "cpsrecordby:$config{'cpsrecordby'}\n");
    print(SAVECONF "startdate:$config{'startdate'}\n");
    print(SAVECONF "listdate:$config{'listdate'}\n");
    print(SAVECONF "filecount:$config{'filecount'}\n");
    print(SAVECONF "sobig:$config{'sobig'}\n");
    close(SAVECONF);
    rename "$config{'confpath'}.new", "$config{'confpath'}";
    return 1;
}


sub sky_help()
{
    sky_print("\00314,8\0032 Working comands: \003");
    sky_print("\00314,8\0034 /skyhelp \0032Display this message \003");
    sky_print("\00314,8\0034 /settings \0032Display the Current Settings \003");
    sky_print("\00314,8\0034 /settings add <key> <new value> \0032Add <new value> to <key> \003");
    sky_print("\00314,8\0034 /settings del <key> <known value> \0032Del <known value> from <key> \003");
    sky_print("\00314,8\0034 /settings del <key> \0032Remove any value(s) from <key> \003");
    sky_print("\00314,8\0034 /settings change <key> <new value> \0032Set <key> to <new value> \003");
    sky_print("\00314,8\0034 /settings default \0032Set default settings \003");
    sky_print("\00314,8\0034 /loadconf \0032Reload config \003");
    sky_print("\00314,8\0034 /saveconf \0032Save config \003");
    sky_print("\00314,8\0034 /createlist \0032Create a new list from offerdirs \003");
    sky_print("\00314,8\0034 /play trigger \0032Play your trigger to the channels you serve \003");
    sky_print("\00314,8\0034 /sendnext \0032Send next file if all rules are true, e.g. a sendslot is available \003");
    sky_print("\00314,8\0034 /skyon \0032Turn Sky-Jukebox on.\003");
    sky_print("\00314,8\0034 /slyoff \0032Turn Sky-Jukebox off.\003");
    return 1;
}


sub sky_play()
{
    my $input = shift;
    if ($input eq "trigger")
    {
	my $notice1 = "\003$config{'background_color'},2\003$config{'script_color'}*** " . $config{'connection_type'} . "-Connection *** For my List (" . $config{'filecount'} . " Files/" . $config{'sobig'} . " GB/" . $config{'listdate'} . ") type " . $config{'triglist'} . " .";
	my $notice2 = " \003$config{'background_color'},2\003$config{'script_color'}To request a file copy and paste the hole line to a channel I am serving in.";
	my $message = $notice1 . $notice2;

	foreach my $channel (@{$config{'channels'}})
        {
            srv_command("/msg $channel $message");
        }
    }
    return 0;
}


sub sky_print_settings()
{
    sky_print("\00314,8\0032Current Settings:\003");
      sky_print(" autovoice_channels: @{$config{'autovoice_channels'}}");
      sky_print(" autovoice_status: $config{'autovoice_status'}");
      sky_print(" channels: @{$config{'channels'}}");
      sky_print(" connection_type: $config{'connection_type'}");
      sky_print(" locator_channels: @{$config{'locator_channels'}}");
      sky_print(" locator_status: $config{'locator_status'}");
      sky_print(" maxQPN: $config{'maxQPN'}");
      sky_print(" maxQueues: $config{'maxQueues'}");
      sky_print(" maxSPN: $config{'maxSPN'}");
      sky_print(" maxSends: $config{'maxSends'}");
      sky_print(" mynickname: $config{'mynickname'}");
      sky_print(" netsplitlimit: $config{'netsplitlimit'}");
      sky_print(" noffersubdirs: @{$config{'noffersubdirs'}}");
      sky_print(" offerdirs: @{$config{'offerdirs'}}");
      sky_print(" onjoin_channels: @{$config{'onjoin_channels'}}");
      sky_print(" onjoin_status: $config{'onjoin_status'}");
      sky_print(" send_raw_status: $config{'send_raw_status'}");
      sky_print(" server_status: $config{'server_status'}");
      sky_print(" show_filesize_in_list: $config{'show_filesize_in_list'}");
      sky_print(" show_details_in_list: $config{'show_details_in_list'}");
      sky_print(" timer: $config{'timer'}");
      sky_print(" slotstimer: $config{'slotstimer'}");
      sky_print(" ziplist: $config{'ziplist'}");
      sky_print(" debug: $config{'debug'}");
      sky_print("\00314,8\0032Color Settings:\003");
      sky_print(" background_color:$config{'background_color'}");
      sky_print(" script_color:$config{'script_color'}");
      sky_print("\00314,8\0032Internal settings:\003");
      sky_print(" filecount: $config{'filecount'}");
      sky_print(" filecounter: $config{'filecounter'}");
      sky_print(" mbsent: $config{'mbsent'}");
      sky_print(" cpsrecord: $config{'cpsrecord'}");
      sky_print(" cpsrecordby: $config{'cpsrecordby'}");
      sky_print(" sobig: $config{'sobig'}");
      sky_print(" startdate: $config{'startdate'}");
      sky_print(" triglist: $config{'triglist'}");
      sky_print(" trigsong: $config{'trigsong'}");
      return 1;
}


sub sky_settings_handler ()
{
    my $line = shift; #get the argument
    my @between = "";
    my (@input) = split(" ", $line);
    if ($input[0] eq "")
    {
	&sky_print_settings;
    }
    elsif ($input[0] eq "add") {
        $input[1] eq "" and do {
            sky_print("\00314,8\0032 Please add <key> and <value>! Type \0034/skyhelp \0032to get help! \003");
          };
        "@allkeys" !~ /\Q$input[1]\E/i and do {
            sky_print("\00314,8\0032 Please choose an existant <key>! Type \0034/skyhelp \0032to get help! \003");
          };
        "@arrays" !~ /\Q$input[1]\E/i and do {
            sky_print("\00314,8\0032 You cant change this <key> with this command, use 0034/settings change <key> <new value> \0032to set <key> to <new value> \003");
          };
        "@arrays" =~ /\Q$input[1]\E/i and do {
            if ($input[2] eq "") {
                sky_print("\00314,8\0032 Please choose a value to add! Type \0034/skyhelp \0032to get help! \003");
              }
            elsif ($input[1] eq "offerdirs" && "@{$config{'offerdirs'}}" !~ /$input[2]/)
            {
                push(@{$config{'offerdirs'}},$input[2]);
            }
            elsif ($input[1] eq "noffersubdirs" && "@{$config{'noffersubdirs'}}" !~ /$input[2]/)
            {
                push(@{$config{'noffersubdirs'}},$input[2]);
            }
            elsif ($input[1] eq "channels" && "@{$config{'channels'}}" !~ /$input[2]/)
            {
                push(@{$config{'channels'}},$input[2]);
            }
            elsif ($input[1] eq "onjoin_channels" && "@{$config{'onjoin_channels'}}" !~ /$input[2]/)
            {
                push(@{$config{'onjoin_channels'}},$input[2]);
            }
            elsif ($input[1] eq "autovoice_channels" && "@{$config{'autovoice_channels'}}" !~ /$input[2]/)
            {
                push(@{$config{'autovoice_channels'}},$input[2]);
            }
            elsif ($input[1] eq "locator_channels" && "@{$config{'locator_channels'}}" !~ /$input[2]/)
            {
                push(@{$config{'locator_channels'}},$input[2]);
            }
            else
            {
                sky_print("\00314,8\0032 Sorry, $input[1] already contains $input[2]! \003");
              }
            sky_print("\00314,8\0032 Added $input[2] to $input[1]! \003");
        }
    }
    elsif ($input[0] eq "del") {
        "@allkeys" !~ /\Q$input[1]\E/i and do {
            sky_print("\00314,8\0032 Please choose an existant <key>. Type \0034/skyhelp \0032to get help! \003");
          };
        "@allkeys" =~ /\Q$input[1]\E/i and do {
            if ($input[1] eq "offerdirs") {
                foreach my $dir (@{$config{'offerdirs'}}) {
                    push(@between,$dir) if $dir ne $input[2];
                }
                ##@{$config{'offerdirs'}} = "";
                @{$config{'offerdirs'}} = @between;
            }
            elsif ($input[1] eq "noffersubdirs") {
                foreach my $dir (@{$config{'noffersubdirs'}}) {
                    push(@between,$dir) if $dir ne $input[2];
                }
                ##@{$config{'noffersubdirs'}} = "";
                @{$config{'noffersubdirs'}} = @between;
            }
            elsif ($input[1] eq "channels") {
                foreach my $chan (@{$config{'channels'}}) {
                    push(@between,$chan) if $chan ne $input[2];
                }
                #@{$config{'channels'}} = "";
                @{$config{'channels'}} = @between;
            }
            elsif ($input[1] eq "onjoin_channels") {
                foreach my $chan (@{$config{'onjoin_channels'}}) {
                    push(@between,$chan) if $chan ne $input[2];
                }
                ##@{$config{'onjoin_channels'}} = "";
                @{$config{'onjoin_channels'}} = @between;
            }
            elsif ($input[1] eq "autovoice_channels") {
                foreach my $chan (@{$config{'autovoice_channels'}}) {
                    push(@between,$chan) if $chan ne $input[2];
                }
                ##@{$config{'autovoice_channels'}} = "";
                @{$config{'autovoice_channels'}} = @between;
            }
            elsif ($input[1] eq "locator_channels") {
                foreach my $chan (@{$config{'locator_channels'}}) {
                    push(@between,$chan) if $chan ne $input[2];
                }
                ##@{$config{'locator_channels'}} = "";
                @{$config{'locator_channels'}} = @between;													              }
        };
        sky_print("\00314,8\0032 If \0034$input[2] \0032was part of \0034$input[1] \0032it has been removed! \003");
    }
    elsif ($input[0] eq "change") {
        "@allkeys" !~ /\Q$input[1]\E/i and do {
            sky_print("\00314,8\0032 Please choose an existant <key>. Type \0034/skyhelp \0032to get help! \003");
          };
        if ("@allkeys" =~ /\Q$input[1]\E/i) {
            $input[2] eq "" and do {
                sky_print("\00314,8\0032 Please choose a value! Type \0034/skyhelp \0032to get help! \003");
              };
            $input[2] ne "" and do {
                $input[1] eq "mynickname" and do {
                    $config{'mynickname'} = $input[2];
                };
                if ($input[1] eq "maxSends") {
                    if ($config{'maxSends'} < $input[2]) {
                        $config{'maxSends'} = $input[2];
                        &sky_sendNext;
                    }
                    else {
                        $config{'maxSends'} = $input[2];
                    }
                } else {
                    $config{$input[1]} = $input[2];
                }
                sky_print("\00314,8\0032 Changed $input[1] to $input[2]! \003");
            };
        }
    }
    elsif ($input[0] eq "default")
    {
	$config{'mynickname'} = 'yournick';
	@{$config{'offerdirs'}} = '/mp3';
	@{$config{'noffersubdirs'}} = '/ftp/download';
	@{$config{'channels'}} = ('#jazzmp3');
	$config{'maxSends'} = 1;
	$config{'maxQueues'} = 10;
	$config{'timer'} = 300000;
	$config{'slotstimer'} = 180000;
	$config{'maxQPN'} = 3;
	$config{'maxSPN'} = 1;
	$config{'onjoin_status'} = 'off';
	@{$config{'onjoin_channels'}} = ('#jazzmp3');
	$config{'autovoice_status'} = 'on';
	@{$config{'autovoice_channels'}} = ('#jazzmp3');
	$config{'locator_status'} = 'on';
	@{$config{'locator_channels'}} = ('#jazzmp3');
	#	$server_status = 'on';
	$config{'ziplist'} = 'on';
	$config{'debug'} = 0;
	$config{'show_filesize_in_list'} = 'off';
	$config{'show_details_in_list'} = 'off';
	$config{'send_raw_status'} = 'on';
	$config{'netsplitlimit'} = 60000;
	$config{'connection_type'} = 'DAILUP';
	$config{'background_color'} = '14';
	$config{'script_color'} = '8';
	#Sky-Jukebox Settings
	$config{'filecounter'} = 0;
	$config{'mbsent'} = 0;
	$config{'cpsrecord'} = 0;
	$config{'cpsrecordby'} = 0;
	$config{'startdate'} = 'notset';
	$config{'listdate'} = 'notset';
	$config{'filecount'} = 0;
	$config{'sobig'} = 0;
    }
    return 1;
}


#creates a list of all files in $offerdirs and adds the $trigsong at the beginning of every line
#creates a list with the paths to the files in $offerdirs
sub sky_create_list()
{
    $config{'server_status'} = 'off';
    my $delistpath = $config{'confdir'} . $config{'mynickname'} . "*";
    if (unlink $delistpath != 1)  {
        sky_print("Cannot unlink $config{'mynickname'} 's list, creating new list!");
    }
    my $date = `date`; #gets the date
    substr($date,-1,1) = ""; #cuts \n off the end
    $config{'listdate'} = `date +%d-%m-%Y`;
    substr($config{'listdate'},-1,1) = "";
    $config{'listpath'} = $config{'confdir'} . $config{'mynickname'} . "-" . $config{'listdate'} . ".txt"; #gets the offerlist-path and selects the filename
    $config{'listofpath'} = $config{'confdir'} . $config{'mynickname'} . "Path-" . $config{'listdate'} . ".txt"; #gets the pathlist-path and selects the filename
    $config{'ziplistpath'} = $config{'confdir'} . $config{'mynickname'} . "-" . $config{'listdate'} . ".zip";
    my $sum = 0;
    my $plus = 0;
    my $minus = 0;
    my ($input) = @_;
    my @newlist = ();
#    if ($input eq "")
#      {
    sky_print("\00314,8\0032 Creating a new list from offerdirs ... \003");
    foreach my $offerdir (@{$config{'offerdirs'}})
    {
        my @between = `find "$offerdir" -type f -follow`; #add all linked files in offerdir to @list
        push(@newlist, @between);
        my @howbig = `du -D "$offerdir"`;
        my @crash = split(" ",pop(@howbig));
        $plus += $crash[0];
    }
    sky_print("\00314,8\0032 ... sorting\003");
    my @xlist = sort { 
        my (@a, @b, $res);
        $a =~ /^(.+)\/([^\/]+)$/;
        @a = ($1, $2);
        $b =~ /^(.+)\/([^\/]+)$/;
        @b = ($1, $2);
        $res = ($a[0] cmp $b[0]);
        $res = ($a[1] cmp $b[1]) unless ($res);
        ##sky_print("Comparing @a .... @b =  $res");
        $res;
    } @newlist;
    @newlist = @xlist;

    foreach my $noffer (@{$config{'noffersubdirs'}}) {
        my @howbig = `du -D "$noffer"`;
        my @crash = split(" ",pop(@howbig));
        $minus += $crash[0];
    }
    sky_print("\00314,8\0032 ... writing (and generating ::INFO:: part)\003");
    $sum = $plus - $minus;
    $config{'sobig'} = sprintf("%.2f", ($sum / 1000000)); #SPEICHERN!!!
    $config{'filecount'} = $#newlist+1; #SPEICHERN!!!
    open(OFFERLIST, ">$config{'listpath'}") or sky_error("Cant open listpath in create_list") and return;# die;
    open(PATHLIST, ">$config{'listofpath'}") or sky_error("Cant open listofpath in create_list") and return;# die;
    my $separator   = "-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-";
    my $instruction = "\n \t \t |*|*| List of " . $config{'mynickname'} . "'s files [" . $config{'sobig'} . " Gigs in " . $config{'filecount'} . " Files] |*|*|\n\n\n Last Updated on " . $date . " by Sky-Jukebox-" . $sky_version . "\n To grab a file from me, type !Mynick Exact_Filename or copy/paste a corresponding line in Channel\n e.g., " . " $config{'trigsong'} " . " Bill_Evans_-_01_-_Granadas.mp3\n\n$separator\n";
    print OFFERLIST $instruction;
    my $oldpath = "";
    foreach my $line (@newlist) #for every line of @list do
    {
        foreach my $noffer (@{$config{'noffersubdirs'}})
        {
            #      sky_print("$noffer bla");
            #      sky_print("$line bla");
            if ($line =~ /$noffer/)
            {
            }
            else {
                print PATHLIST $line; #save the line to file_handler PATHLIST
                my (@broken) = split /\//, $line; #spilt at /
                my $file = pop(@broken); #gets the late part of @broken, the filename.mp3
                my $newpath = join('/',@broken);
                if ($oldpath ne $newpath)
                {
                    my $pathline = "\n" . "------ [" . "$newpath" . "] ------" . "\n" . "\n";
                    print OFFERLIST $pathline;
                    $oldpath = $newpath;
                }
                my $newline;
                if ($file ne "")
                {
                    chomp($file);
                    my $fullfilepath = $newpath."/".$file; # make complete path
                    if ($config{'show_filesize_in_list'} eq "on") {
                        my $sizestr = (stat($fullfilepath))[7];
                        $file = sprintf("%-80s ::INFO:: %.1fMB",$file,$sizestr/(1024*1024)); # make the size-string
                    }
                    if ($config{'show_details_in_list'} eq 'on') {
                        if ($file =~ /mp3/i) {
                            my $tag = new MP3::Info $fullfilepath;
                            $file .= " $tag->{TIME}";
                            if ($tag->{VBR}) {
                                $file .= " VBR ($tag->{BITRATE}kbps avg)";
                            } else {
                                $file .= " $tag->{BITRATE}kbps";
                            }
                            $file .= " $tag->{FREQUENCY}kHz";
                            if ($tag->{MODE} == 0) {
                                $file .= " Stereo";
                            } elsif ($tag->{MODE} == 1) {
                                $file .= " Joint Stereo";
                            } elsif ($tag->{MODE} == 2) {
                                $file .= " Dual Channel";
                            } elsif ($tag->{MODE} == 3) {
                                $file .= " Single Channel";
                            }
                        } elsif ($file =~ /ogg/i) {
                            my $DDD = `/home/saschal/language/c/ogginfo/ogginf XXXa "$fullfilepath"`;
                            $DDD =~ /(\d+) Hz +(\d+) KB\/s +([\d:]+)/;

                            $DDD =~ /total_playtime=([\d:]+)/;
                            $file .= sprintf(" %s %dkbps %3.1fkHz", $3, $2, $1 / 1000);
                        }
                    }
                    
                    $newline = $config{'trigsong'} . " $file\n"; #add the songtrigger at the beginning of current line
                    print OFFERLIST $newline; #save the line to file_handler OFFERLIST
                }
            }
        }
    }
    print OFFERLIST "\n$separator\nEnd of List - Total: $config{'filecount'} files - $config{'sobig'}GB\n";
    close(PATHLIST); #closes PATHLIST
    close(OFFERLIST); #closes OFFERLIST
    sky_print("\00314,8\0032 New list created.\003");
    if ($config{'ziplist'} eq "on")
    {
	chdir "$config{'confdir'}" or sky_error("Couldnt change in your confdir") and return;# die;
	my $zipfile = $config{'mynickname'} . "-" . $config{'listdate'};
	my $tozipfile = $config{'mynickname'} . "-" . $config{'listdate'} . ".txt";
        `zip "$zipfile" "$tozipfile"` or sky_error("Couldnt zip the list in create_list") and return;# die;
    }
    if (open(CONTENT, "<$config{'listofpath'}")) {
        while (<CONTENT>){
            ## [TR FIX] (my $underline_filename = $_) =~ tr/ /_/;
            my $underline_filename = $_;
            my (@broken) = split /\//, $underline_filename; #spilt at /
            my $file = pop(@broken); #gets the late part of @broken, the filename.mp3
            chop $file;
            $files_with_path{$file} = $_;
#	$locator_hash{lc $file} = $_;
        }
        open(LISTE, "<$config{'listpath'}");
        while (<LISTE>){
            ## [TR FIX] (my $filename = $_) =~ tr/ /_/;
            my $filename = $_;
            $filename = substr($filename, length($config{'trigsong'})+1);
            $locator_hash{lc $filename} = $_;
        }
        close(LISTE);
        close(CONTENT);
    }
    else {
        sky_print("Cant open a listfile, please run createlist again!");
      }
    $config{'server_status'} = 'on';
    &sky_save_conf();
    return 1;
}



sub sky_search_list
{
    sky_debug("search_list entered: @_");
    my $newmessage = shift; #get the argument
    ## [TR FIX] $newmessage =~ tr/ /_/;
    my $searchresult;

    $searchresult = $files_with_path{$newmessage};
    sky_debug("SR1: $searchresult");
    if ($searchresult eq '') { 
        my @ll = split(/ /, $newmessage);
        my $key = shift @ll;
        while (@ll) {
            $searchresult = $files_with_path{$newmessage};
            sky_debug("SR2: $searchresult");
            last if ($searchresult ne '');
            $key .= ' ' . shift @ll;
        }
        $searchresult = 'error' if ($searchresult eq '');
    }
    chomp $searchresult;
    sky_debug("SR3: $searchresult");
    return($searchresult); #returns /path/to/file/file.mp3
}



sub sky_publicmsg_handler()
{
    my ($server, $message, $nick, $mask, $current_channel) = @_;

    $message =~ s/^ *(.+?) *$/$1/;
    # strip colorized resume requests from SpR Jukebox
    #$message =~ s/.+//;
    my $removetrigger = $config{'mynickname'} . '-remove';
    my $newmessage = "";
    if ($config{'server_status'} eq "on") {
        if ("@{$config{'channels'}}" =~ /\Q$current_channel\E/i) {
            $message =~ /^\Q$config{'triglist'}\E$/i and do {
                if ($config{'ziplist'} eq "on") {
                    $server->command("/dcc send $nick $config{'ziplistpath'}"); #sends my zipped list to $nick
                }
                else {
                    $server->command("/dcc send $nick $config{'listpath'}"); #sends my list to $nick
                }
            };
            $message =~ /\@sky-jukebox-$sky_version/i and do {
                $server->command("/dcc send $nick $config{'scriptpath'}"); #sends the script to $nick
            };
            $message =~ /\Q$config{'trigsong'}\E/i and do {
                $newmessage = substr($message, length($config{'trigsong'})+1); #cut the songtrigger
                ## [FIX] $newmessage =~ s:(mp3).*:$1:; # cut the ::info
                $newmessage =~ s/(.) +::INFO.+$/$1/; # cut the ::info
                ## [FIX] my $t_and_f = &sky_search_list($newmessage); #checks whether the file is in my list
                ## [FIX] my $filetosend = &sky_pathworking($t_and_f);
                my $filetosend = &sky_search_list($newmessage); #checks whether the file is in my list
                if ($filetosend ne 'error') {
                    &sky_addQueue($nick,$mask,$filetosend,$current_channel);
                } 
                elsif ($filetosend eq "\/\"error\"") {
                    sky_notice_nick($nick, "Unknown command!!! To request my list type " . $config{'triglist'} . "\r\n");
                }
                else {
                    sky_notice_nick($nick, "Sorry, this file isnt in my list or you misspelled it!\r\n");
                }
            };
            $message =~ /$config{'triglist'}-stats/i and do {
                &sky_listSends($nick,"list"); #displays the current sends to $nick
                &sky_listQueues($nick,$mask); #displays the current queues to $nick
            };
            $message =~ /$config{'triglist'}-que/i and do {
                &sky_listQueues($nick,$mask); #displays the current queues to $nick
            };
            $message =~ /$config{'triglist'}-sends/i and do {
                &sky_listSends($nick,"list"); #displays the current sends to $nick
            };
            $message =~ /^.\Q$removetrigger\E/i and do {
                my $queuedFile = substr($message, length($config{'triglist'} . '-remove') +1);
                if ($queuedFile eq "") {
                    &sky_remove_que($nick,$mask,"");
                }
                elsif ($queuedFile =~ /mp3/) {
                    &sky_remove_que($nick,$mask,$queuedFile);
                }
            }
        }
        elsif ($config{'server_status'} eq "off" && $message =~ /^.\Q$config{'mynickname'}\E/i) {
            sky_notice_nick($nick, "Sorry, server is offline!\r\n");
        }
        return 0;
    }
}


sub sky_listSends()
{
    sky_debug("listSends entered:  @_");
    my ($nick, $question) = @_;
    my @dccs = Irssi::Irc::dccs();
    my $counter = 0;
    my $internal_counter = 0;
    my $i = 0;
    foreach my $dcc_object (@dccs)
    {
        #if ($question eq "info" && $dcc_object->{type} eq 'SEND' && !$dcc_object->{waitforend} && $files_with_path{$dcc_object->{arg}}) #[FIX] && $dcc_object->{arg} =~ /\.mp3$/i)
        if ($question eq "info" && $dcc_object->{type} eq 'SEND' && !$dcc_object->{waitforend} && $files_with_path{$dcc_object->{arg}}) #[FIX] && $dcc_object->{arg} =~ /\.mp3$/i)
        {
            $counter++;
        }
        elsif ($question eq "list" && $dcc_object->{type} eq 'SEND' && !$dcc_object->{waitforend} && $files_with_path{$dcc_object->{arg}}) #[FIX] && $dcc_object->{arg} =~ /\.mp3$/i)
        {
            my (@broken) = split /\//, $dcc_object->{arg}; #split at /
            my $file = pop(@broken); #gets the late part of @broken, the filename.mp3
            
            sky_notice_nick($nick, "* Sending $dcc_object->{nick} $file."); ##[FIXME cps]  at $dcc_object->{cps} cps\r\n");
            $counter++;
        }
        elsif ($question eq "SPN" && $dcc_object->{type} eq 'SEND' && !$dcc_object->{waitforend} && $files_with_path{$dcc_object->{arg}}) #[FIX] && $dcc_object->{args} =~ /\.mp3$/i)
        {
            $internal_counter++;
            if ($dcc_object->{nick} eq $nick)
            {
                $counter++;
            }
        }
    }
    if ($question eq "list" && $counter  == 0 && $internal_counter == 0)
    {
        sky_notice_nick($nick, "\003$config{'background_color'},2\003$config{'script_color'}No Files are being sent.\r\n");
    }
    return $counter;
}




sub sky_addQueue()
{
    sky_debug("addQueue entered: @_");
    my ($nick, $mask, $file_to_send, $current_channel) = @_;
    my $slotcount = &sky_listSends("","info");
    my $queuecount = &sky_numQueues();
    my $QPN = &sky_QueuesPerNick($mask);
    $queuecount++;
    if (!($QPN < $config{'maxQPN'})) {
        sky_notice_nick($nick, "Sorry, you used all your queuespots! ($config{'maxQPN'})\r\n");
    }
    elsif ($queuecount > $config{'maxQueues'}) {
        sky_notice_nick($nick, "Sorry, I already have " . $config{'maxQueues'} . " Files in my Queue, please try it again later!\r\n");
    }
    elsif ($QPN < $config{'maxQPN'} && $queuecount < $config{'maxQueues'})
    {
	my $to_add = "$mask" . "" . " $file_to_send ";
	my (@peaces) = split /\//, $file_to_send; #spilt at /
	my $file = pop(@peaces); #gets the late part of @broken, the filename
	my (@check, @allqueues) = ();
	if (open (CHECK, $config{'queuelocation'})) {
            @check = <CHECK>;
            close(CHECK);
        } else {
            @check = ();
        }
        sky_debug("AQ queue is: @check");
        if (!("@check" =~ /\Q$to_add\E/i)) #is the to_add_file already in my queue, no, then add to_add
        {
	    open (QUEUE, "+>>$config{'queuelocation'}") or sky_error("Can't open queuelocation in addQueue") and return;# die;
	    @allqueues = <QUEUE>;
	    push(@allqueues,"$to_add $current_channel\n");
            print QUEUE "@allqueues";
	    close(QUEUE);
	    $QPN++;
            sky_notice_nick($nick, "I have added " . $file . " as " . $queuecount . " in my que. This makes " . $QPN . " of " . $config{'maxQPN'} . " that you are allowed in my que. Type \"" . $config{'triglist'} . "-remove\" with or without filename to remove your file\(s\) from my que, e.g. \"" . $config{'triglist'} . "-remove (Enrique Iglesias) - 02 - Trapecista.mp3\"!\r\n");
        }
	elsif ("@check" =~ /\Q$to_add\E/i) {
            sky_notice_nick($nick, "You already have " . $file . " queued. Please try another one!\r\n");
	}
	&sky_sendNext;
    }
    return 0;
}


sub sky_remove_que()
{
    my ($nick, $mask, $file) = @_;
    my $newlocation = $config{'confdir'} . "queues.bak";
    open(OLDQUEUE, "<$config{'queuelocation'}") or sky_error("Cant open queuelocation in remove_que") and return;# die;
    open(NEWQUEUE, ">$newlocation") or sky_error("Cant open newlocation in remove_que") and return;# die;
    if ($file eq "")
    {
	while(<OLDQUEUE>)
        {
	    print NEWQUEUE unless(/\Q$mask\E/i);
        }
	if ($nick ne "error")
        {
	    sky_notice_nick($nick, "Your Files have been removed from my que!\r\n");
        }
    }
    elsif ($file eq "part")
    {
	while(<OLDQUEUE>)
        {
	    print NEWQUEUE unless(/\Q$mask\E/i);
        }
    }
    elsif ($file ne "")
    {
	while(<OLDQUEUE>)
        {
	    print NEWQUEUE unless(/\Q$mask\E/ && /\Q$file\E/);
        }
        sky_notice_nick($nick, "" . $file . " has been removed from my que!\r\n");
    }
    close(OLDQUEUE);
    close(NEWQUEUE);
    rename "$newlocation","$config{'queuelocation'}";
    return 0;
}


sub sky_numQueues()
{
    my $nr = 0;
    my @allqueues = ();
    if (open (QUEUE, "<$config{'queuelocation'}")) {
        @allqueues = <QUEUE>;
        close(QUEUE);
        foreach my $queue (@allqueues)
        {
            $nr++;
        }
    }
    return $nr;
}


sub sky_listQueues()
{
    my ( $nick , $mask ) = @_;
    my $i = 0;
    my $queuecount = &sky_numQueues();
    if ($queuecount > 0)
    {
	open (QUEUE, "<$config{'queuelocation'}") or sky_error("Cant open queuelocation in listQueues") and return;# die;
	while(<QUEUE>) {
	    $i++ if (/\Q$mask/i);
	    next;
        }
	close(QUEUE);
	if ($i > 0) {
            sky_notice_nick($nick, "You have $i of $config{'maxQPN'} files in my queue!\r\n");
        } else {
            sky_notice_nick($nick, "($queuecount/$config{'maxQueues'}) Available Queues are taken!\r\n");
        }
    }
    else
    {
        sky_notice_nick($nick, "There are no files queued! $config{'maxQueues'} Queueslots available!\r\n");
    }
    return 0;
}


sub sky_QueuesPerNick()
{
    my $mask = shift;
    my $nickcount = 0;
    my @allqueues = ();
    if (open (QUEUE, $config{'queuelocation'})) {
        @allqueues = <QUEUE>;
        close(QUEUE);
        foreach my $queue (@allqueues)
        {
            if ($queue =~ /\Q$mask\E/i)
            {
                $nickcount++;
            }
        }
    }
    return $nickcount;
}


sub sky_sendNext()
{
    sky_debug("SN entered: @_");
    my $slotcount = &sky_listSends("","info");
    my $queuecount = &sky_numQueues();
    my $i = 0;
    my (@allqueues, @broken) = ();
#    if ($server_status eq "on")
#    {
    while($slotcount < $config{'maxSends'} && $i < $queuecount)
    {
	if ($queuecount != 0)
        {
	    open(QUEUE, "<$config{'queuelocation'}") or sky_error("Cant open queuelocation in sendNext1") and return;# die;
	    @allqueues = <QUEUE>;
	    close(QUEUE);
	    (@broken) = split(" ", $allqueues[$i]); #split at /
	    my $current_mask = shift(@broken);
	    my $current_channel = pop(@broken);
	    my $nextfile = "@broken";
	    my @userInfo = &sky_listUser($current_mask,$current_channel);
	    if ($userInfo[0] ne "error")
            {
		my $currentSPN = &sky_listSends($userInfo[0],"SPN");
		if ($currentSPN < $config{'maxSPN'})
                {
		    ## [pending] REMOVEME $nextfile = "@broken";
		    splice(@allqueues,$i,1);
		    open(QUEUE, ">$config{'queuelocation'}") or sky_error("Cant open queuelocation in sendNext2") and return;# die;
		    print QUEUE "@allqueues";
		    close(QUEUE);
                    sky_print("Sending to $userInfo[0] ($current_channel): $nextfile");
		    srv_command("/dcc send $userInfo[0] $nextfile");
		    $slotcount++;
                }
		elsif ($currentSPN >= $config{'maxSPN'})
                {
		    $i++;
                }
		$queuecount = &sky_numQueues();
            }
	    else
            {
		splice(@allqueues,$i,1);
		open(QUEUE, ">$config{'queuelocation'}") or sky_error("Cant open queuelocation in sendNext3") and return;# die;
		print QUEUE "@allqueues";
		close(QUEUE);
                sky_print("\00300,04Throwing from queue $current_mask ($current_channel): @broken");
		$queuecount = &sky_numQueues();
            }
        }
    }
#      }
#      else
#      {
#      IRC::command("/notice $nick \00314,2\0038Sorry, server is offline!");
#      }
    return 0;
}


sub sky_timer()
{
    my $slotcount = &sky_listSends("","info");
    my $queuecount = &sky_numQueues();
    my $queuestatus = $queuecount . "/" . $config{'maxQueues'};
    my $slotstatus = $slotcount . "/" . $config{'maxSends'};
    my $trigliststats = $config{'triglist'} . "-stats";
    my $triglistque = $config{'triglist'} . "-que";
    my $example = $config{'trigsong'} . " file.mp3";
    my $notice1 = "\003$config{'background_color'},2\003$config{'script_color'}*** " . $config{'connection_type'} . "-Connection *** For my List (" . $config{'filecount'} . " Files/" . $config{'sobig'} . " GB/" . $config{'listdate'} . ") type " . $config{'triglist'} . " .";
    my $notice2 = "\003$config{'background_color'},2\003$config{'script_color'} To see my stats and queues type " . $trigliststats . " and " . $triglistque . " .";
    my $notice3 = "\003$config{'background_color'},2\003$config{'script_color'} (" . $slotstatus . ") Slots and (" . $queuestatus . ") Queues taken.";
    ## my $notice4 = "\003$config{'background_color'},2\003$config{'script_color'} To request a file copy and paste the whole line to a channel I am serving in.";
    my $notice4 = '';
    ## my $notice5 = "\003$config{'background_color'},2\003$config{'script_color'} (like " . $example . ")";
    my $notice5 = '';
    my $gsent = sprintf("%3.2f", $config{'mbsent'} / 1024 );
    my $notice6 = "\003$config{'background_color'},2\003$config{'script_color'} "
        . $config{'filecounter'} . " Files ($gsent G) sent since " . $config{'startdate'};
    if ($config{cpsrecord}) {
        $notice6 .= " (CPS record: $config{cpsrecord})."; ## by $config{cpsrecordby}).";
    } else {
        $notice6 .= " (no CPS record so far).";
    }
    my $trigger_message = $notice1 . $notice2 . $notice3 . $notice4 . $notice5 . $notice6;
    $trigger_message .= " -- \00304,00This is Sky-Jukebox $sky_version (port of Sky-Skript) for Irssi (Linux IRC client).";
    $trigger_message .= "  Say \@Sky-Jukebox-$sky_version to get it." if $config{scriptpath};
    $trigger_message .= " -- No, notepad cannot read my list.";
    my @keys = keys %files_with_path;
    my $randomnumber = int(rand @keys);
    my $song_to_play = $files_with_path{$keys[$randomnumber]};
    my (@broken) = split /\//, $song_to_play;
    $song_to_play = pop(@broken);
    chop($song_to_play);
    foreach my $channel (@{$config{'channels'}})
    {
	srv_command("/msg $channel $trigger_message");
	srv_command("/ctcp $channel MP3 $song_to_play");
    }
    return 0;
}


sub sky_slots_timer
{
    my $slotcounts = &sky_listSends("","info");
    my $queuecount = &sky_numQueues();
    my $slotsfree = $config{maxSends} - $slotcounts;
    my $queuesfree = $config{maxQueues} - $queuecount;
    my $nextslot;

    # calctime next slot will be free
    if ($slotsfree) {
        $nextslot = "NOW";
    } else {
        my $timeleft = 4_294_967_296; # ~138 years as max value to start from
        foreach my $dcc (Irssi::Irc::dccs()) {
            if ($dcc->{type} eq 'SEND') {
                my $file = $dcc->{arg};
                if (exists $files_with_path{$file}) {
                    my $passed = time - $dcc->{starttime}; 
                    my $trans = ($dcc->{transfd} ? $dcc->{transfd} : 1);
                    my $time = $passed * ($dcc->{size}/$trans) - $passed;
                    $timeleft = $time if $time < $timeleft;
                }
            }
        }
        my @tl = localtime $timeleft;
        my $days = $tl[3] - 1;
        $nextslot = ($days? $days . 'd' : '') 
            . ($tl[2]? "$tl[2]h" : '')
            . ($tl[1]? "$tl[1]m" : '')
            . ($tl[0]? "$tl[0]s" : '');
    }


    foreach my $channel (@{$config{'channels'}})
    {
        ## CTCP SLOTS MAX FREE NEXT QUEUE FREE SPEED FILES
        srv_command("/ctcp $channel SLOTS $config{maxSends} $slotsfree $nextslot $queuesfree $config{maxQueues} N/A $config{filecount}");
    }
}

## [FIX] obsolete?
sub sky_pathworking()
{
    my $t_and_f = shift;
    my (@broken) = split /\//, $t_and_f; #split at /
    my $file = pop(@broken); #gets the late part of @broken, the filename.mp3
    my $path = join('/',@broken); #fix the path with / between the pieces
    $path = $path . '/'; #add / at the end
    my $file_to_send = $path  . '"' . $file . '"'; #add " " - spaces allowed now
    return $file_to_send;
}



# sub sky_get_join_data()
#   {
#    my $line = shift; #get the argument
#    $line =~ /:([^!]*)!([^@]*@[^ ]*) [^ ]*[\s][^:]*:(.*)/;
#    my ($nick,$mask,$channel) = ($1,$2,$3);
#    return ($nick,$mask,$channel); #return nick, mask and channel
#  }


#sub sky_get_part_data()
# {
#    my $line = shift; #get the argument
#    $line =~ /:([^!]*)!([^@]*@[^ ]*) [^ ]*[\s]([^:]\S*).*/;
#    @{$config{'part_remove'}} = ($1, $2, $3);
#  }


# sub sky_join_handler()
#   {
#     my ($nick, $mask, $channel) = &sky_get_join_data;
#     my $check = &sky_check_channel($channel);
#     if ($config{'server_status'} ne "on" && $check == "1" && $nick ne $config{'mynickname'})
#       {
#	IRC::command("/msg $nick \003$config{'background_color'},2\003$config{'script_color'}Hello " . $nick . ", welcome to " . $channel . "! Enjoy your stay here and add this channel to your ajoin-list!!!");
#       }
#     elsif ($config{'server_status'} eq "on" && $check == "1" && $nick ne $config{'mynickname'})
#       {
#	IRC::command("/msg $nick \003$config{'background_color'},2\003$config{'script_color'}Hello " . $nick . ", welcome to " . $channel . "! To get my list type \"" . $config{'triglist'} . "\", to request a song \"" . $config{'trigsong'} . " songname.mp3\" in " . $channel . ". Enjoy your stay here and add this channel to your ajoin-list!!!");
#       }
#     return 0;
#   }


#sub sky_part_handler()
#  {
#    &sky_get_part_data;
#    my @allqueues = ();
#    open(QUEUE, "<$queuelocation") or sky_error("Cant open queuelocation in part_handler") and return;# die;
#    @allqueues = <QUEUE>;
#   close(QUEUE);
#    my $secs = $netsplitlimit/1000;
#    if ("@allqueues" =~ /\Q$part_remove[1]\E/i)
#      {
#	IRC::command("/notice $part_remove[0] \00314,2\0038Rejoin " . $part_remove[2] . " within " . $secs . " secs or you will be removed from my que!!!");
#	IRC::add_timeout_handler($netsplitlimit,"sky_part_remove");
#      }
#      return 0;
#  }


#sub sky_quit_handler()
#  {
#    my $line = shift; #get the argument
#    my ($quit_nick , $quit_mask) = ($line =~ /:([^!]*)!([^@]*@[^ ]\S*) [^ ]*[\s][^:]*:.*/);
#    my @dcc_list = IRC::dcc_list(); #gets dcc_list from IRC
#    my $i = 0;
#    my @allqueues = ();
#    open(QUEUE, "<$queuelocation") or sky_error("Cant open queuelocation in quit_handler") and return;# die;
#    @allqueues = <QUEUE>;
#    close(QUEUE);
#    my $secs = $netsplitlimit/1000;
#    if ("@allqueues" =~ /\Q$quit_mask\E/i)
#      {
#	&sky_remove_que($quit_nick,$quit_mask,"part");
#      }
#    while($i<$#dcc_list)
#      {
#	my $dcc_object = {
#			  nick => $dcc_list[$i],
#			  file => $dcc_list[++$i],
#			  type => $dcc_list[++$i],
#			  stat => $dcc_list[++$i],
#			  cps => $dcc_list[++$i],
#			  size => $dcc_list[++$i],
#			  resumable => $dcc_list[++$i],
#			  addr => $dcc_list[++$i],
#			  destfile => $dcc_list[++$i],
#			 };
#	++$i;
#                sky_print("nick: $dcc_object->{nick}");
#                sky_print("file: $dcc_object->{file}");
#                sky_print("type: $dcc_object->{type}");
#                sky_print("stat: $dcc_object->{stat}");
#                sky_print("cps: $dcc_object->{cps}");
#                sky_print("size: $dcc_object->{size}");
#                sky_print("resumable: $dcc_object->{resumable}");
#                sky_print("addr: $dcc_object->{addr}");
#                sky_print("destfile: $dcc_object->{destfile}");
#	if ($dcc_object->{stat} < 3 && $dcc_object->{nick} =~ /\Q$quit_nick\E/i)
#	  {
#	    IRC::command("/dcc close send $quit_nick");
#	     }
#      }
#    return 0;
#  }



#sub sky_part_remove
#  {
#    my $server = IRC::get_info(3);
#    my $i = 0;
#    my @user_list = IRC::user_list($part_remove[2],$server); #gets dcc_list from IRC
#	while($i<$#user_list)
#	{
#		my $user_object = {
#			nick => $user_list[$i],
#			mask => $user_list[++$i],
#			op => $user_list[++$i], #1 or 0
#			voice => $user_list[++$i], #1 or 0
#			split => $user_list[++$i], #the data of the users are split with :
#		};
#		++$i;
#		sky_print("nick: $user_object->{nick}");
#                sky_print("mask: $user_object->{mask}");
#                sky_print("op: $user_object->{op}");
#                sky_print("voice: $user_object->{voice}");
#     }
#    if ("@user_list" !~ /\Q$part_remove[1]\E/i)
#      {
#	&sky_remove_que($part_remove[0],$part_remove[1],"part");
#        IRC::command("/dcc close send $part_remove[1]");
#      }
#    return 0;
#  }


# sub sky_check_channel
#   {
#     my $chan = $_[0];
#     if (${config{'onjoin_status'}} eq 'on') {
#     foreach my $element (@{$config{'onjoin_channels'}})
#       {
#	if ($element eq $chan)
#           {
#             my $result = 1;
#             return $result;
#           }
#       }
#   }
# return 0;
# }



sub sky_locator_handler
{
    my $i = 0;
    my $end = 3;

    my ($server, $message, $nick, $addr, $current_channel) = @_;

    my $fts = ();
    if ($config{'locator_status'} eq "on" && "@{$config{'locator_channels'}}" =~ /\Q$current_channel\E/i && $message =~ /^.(locator|find)\s/) {
        $message =~ /^.(locator|find)\s(.+)$/;
        my $fts = $2;
        my $request = $fts;
        $request =~ s/^\s*(.+)\s*$/$1/g;
        $request =~ s/^[*+?\\]*(.+?)[*+?\\]*$/$1/;
        $request = quotemeta $request;
        $request =~ s/\\\*/.*/g;
        my @result = ();
        foreach my $key (sort keys %locator_hash) {

            push (@result, $locator_hash{$key}) if $key =~ /$request/i;
        }
        foreach my $result (@result)
	{
            $result =~ s/(.) +::INFO.+$/$1/; ## [FIX filetypes] s:(mp3).*:$1:;
	}
        my $len = $#result+1;
        if ($len > 0 && $len < $end+1)
	{
            sky_notice_nick($nick, "Matches for $fts Copy and Paste in the Channel to Request a File:\r\n");
            foreach my $result (@result)
	    {
                ## [FIX filetypes] if ($result =~ /mp3/)
		##{
                my (@broken) = split /\//, $result; #spilt at /
                my $file = pop(@broken); #gets the late part of @broken, the filename.mp3

                chop $file;
                sky_notice_nick($nick, "   $result\r\n");
		## }
	    }
            sky_notice_nick($nick, "To request my list type $config{'triglist'} in a channel I am serving in.\r\n");
	}#if
            elsif ($len > $end)
	{
            sky_notice_nick($nick, "Matches for $fts Copy and Paste in the Channel to Request a File:\r\n");
            while ($i < $end)
	    {
                if ($result[$i] !~ /^------/)
		{
                    my (@broken) = split /\//, $result[$i]; #spilt at /
                    my $file = pop(@broken); #gets the late part of @broken, the filename.mp3
                    chop $file;
                    #		  $result[$i] = "$config{'trigsong'}" . " " . "$file";
                    sky_notice_nick($nick, "   $result[$i]\r\n");
                }
                else
		{
                    $end++;
                    my $now = $len;
                    $len = $now-1;
		}
                $i++;
	    }#while
                sky_notice_nick($nick, "Too many results [$len] to display! Request my list typing $config{'triglist'} in a channel I am serving in.\r\n");
	}
    }
    elsif ($config{'locator_status'} ne "on" && "@{$config{'locator_channels'}}" =~ /\Q$current_channel\E/i && $message =~ /^.(locator|find)\s/) {
        sky_notice_nick($nick, "Sorry, \@locator is OFF! Request my list typing $config{'triglist'} in a channel I am serving in.\r\n");
    }
    return 0;
}



sub sky_filecounter
{
    sky_debug("filecounter entered: @_");
    my $dcc = shift;

    if ($dcc->{type} eq 'SEND') {
        if ($dcc->{size} == $dcc->{transfd}) {
            $config{'filecounter'}++;
            $config{'mbsent'} += $dcc->{size} / (1024*1024);
            my $timepassed = time - $dcc->{starttime};
            $timepassed = 1 unless $timepassed;
            my $cps = ($dcc->{size} - $dcc->{skipped}) / $timepassed;
            if (!$config{cpsrecord} || $cps > $config{cpsrecord}) {
                $config{cpsrecord} = sprintf ("%2.1f", $cps);
                $config{cpsrecordby} = $dcc->{nick};
            }
            &sky_save_conf();
        }
        &sky_sendNext;
    }
    return 0;
}


sub sky_privmsg_to_mynick_handler
{
    my ($server, $message, $nick, $addr) = @_;

    if ($message =~ /^.\Q$config{'mynickname'}\E/i)  {
	sky_notice_nick($nick, "Please type the request in channel. If you did already, please type alt+o, go to sound(/request), uncheck \"send !nick as a priv msg\".\r\n");
    }
    return 0;
}



#sub sky_autovoice()
#  {
#    my $line = shift; #get the argument
#    $line =~ /:[^!]*!([^@]*@[^ ]*) [^ ]*[\s]([^:]*):(.*)/;
#    my $mask = $1; #who has send the message.
#    my $current_channel = $2; #channel where msg was typed in
#    chop $current_channel; #removes the last sign of $current_channel
#    my $message = $3; #what was the message.
#    if ($autovoice_status eq "on")
#      {
#	my @userInfo = &sky_listUser($mynickname,$current_channel);
#	if ($userInfo[2] == 1)
#	  {
#	    if ("@autovoice_channels" =~ /\Q$current_channel\E/i && $message =~ /MP3/ || "@autovoice_channels" =~ /\Q$current_channel\E/i && $message =~ /SOUND/ || "@autovoice_channels" =~ /\Q$current_channel\E/i && $message =~ /SLOTS/)
#	      {
#		my @user_object = &sky_listUser($mask,$current_channel);
#		if ($user_object[1] eq $mask && $user_object[2] != 1 && $user_object[3] != 1)
#		  {
#		    IRC::command("/mode $current_channel +v $user_object[0]");
#		    IRC::command("/msg $current_channel \00314,2\0038Thank you $user_object[0] for playing your tunes here in $current_channel!!! Thank You for sharing !");
#		  }
#	      }
#	  }
#      }
#      return 0;
#  }




sub sky_listUser()
{
    sky_debug("listUser entered: @_");
    my ($mask, $channel) = @_;
    my $i = 0;
    my @user_list = Irssi::Channel::nicks(srv_channel_find($channel));

    while($i<$#user_list)
    {
        my $user_object = $user_list[$i];
        #sky_print("userinfo  nick: $user_object->{nick} host: $user_object->{host}");
	if ($mask eq $config{'mynickname'})
        {
	    if ($user_object->{nick} eq $mask)
            {
		my @userInfo = ($user_object->{nick}, $user_object->{host}, $user_object->{op}, $user_object->{voice});
		return @userInfo;
            }
        }
	elsif ($user_object->{host} eq $mask)
        {
	    my @userInfo = ($user_object->{nick}, $user_object->{host}, $user_object->{op}, $user_object->{voice});
	    return @userInfo;
        }
	++$i;
    } #while
    my @userInfo = ("error", $mask);
    return @userInfo;
}

# # sub sky_clear_sends()
# #   {
# #     my $what_to_clear = shift; #get the argument
# #     my @dccs = Irssi::Irc::dccs();
# #     my $i = 0;
# #     while($i<$#dccs) {
# #         my $dcc = $dcc[++$i];
# # #       my $dcc_object = {
# # #			nick => $dcc_list[$i],
# # #			file => $dcc_list[++$i],
# # #			type => $dcc_list[++$i],
# # #			stat => $dcc_list[++$i],
# # #			cps => $dcc_list[++$i],
# # #			size => $dcc_list[++$i],
# # #			resumable => $dcc_list[++$i],
# # #			addr => $dcc_list[++$i],
# # #			destfile => $dcc_list[++$i],
# # #		       };
# # #       ++$i;
# #       if ($what_to_clear !~ /failed|done/i && $dcc_object->{type} == 0 && $dcc_object->{stat} != 1) {
# #	Irssi::command("/dcc close send $dcc_object->{nick} \"$dcc_object->{file}\"");
# #       }
# #       elsif ($what_to_clear =~ /failed/i && $dcc_object->{type} == 0 && $dcc_object->{stat} == 2) {
# #	Irssi::command("/dcc close send $dcc_object->{nick} \"$dcc_object->{file}\"");
# #       }
# #       elsif ($what_to_clear =~ /done/i && $dcc_object->{type} == 0 && $dcc_object->{stat} == 3) {
# #	Irssi::command("/dcc close send $dcc_object->{nick} \"$dcc_object->{file}\"");
# #       }
# #       else {
# #       }
# #     }
# #     return 1;
# #   }



sub sky_offerscript()
{
    my $notice = "\0037,8\0032*** I use \0034Sky-Jukebox $sky_version for Linux! \0032To get this script type \0034\@Sky-Jukebox-$sky_version \0032in a channel I am serving in! *** \003";
    foreach my $channel (@{$config{'channels'}})
    {
        srv_command("/msg $channel $notice");
    }
    return 0;
}

sub sky_notice_handler()
{

    my ($server, $args, $nick, $addr, $target) = @_;

    $args =~ /.*\suser\s(.*)\sis.*\sfiletype\s(.*)\sfrom.*$/;
    my $nick = $1; #who cant receive this filetype
    my $filetype = $2; #filetype

    if ($filetype) {
        sky_notice_nick($nick, "Cause of security reasons, your server blocks all " . $filetype . " files. If you wanna receive my list, type \"/dccallow +" . $config{'mynickname'} . "\" and request it again!\r\n");
    }
    return 0;
}
